package com.wechi.adweb.bridge.google.analytics;

import com.google.analytics.admin.v1beta.*;
import com.google.api.client.util.Lists;
import com.google.api.gax.core.FixedCredentialsProvider;
import com.google.auth.oauth2.GoogleCredentials;
import com.google.common.base.Splitter;
import com.google.gson.Gson;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.GAAccountDTO;
import com.wechi.adweb.bridge.google.analytics.dto.GADataStreamDTO;
import com.wechi.adweb.bridge.google.analytics.dto.GAPropertyDTO;

import jakarta.annotation.PostConstruct;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;

/**
 * @author wfansh
 */
@Slf4j
@Service
public class GAAdminService {

    @Value("${google.service.account.key}")
    private String serviceAccountKey;

    private AnalyticsAdminServiceSettings adminServiceSettings;

    private Gson gson = new Gson();

    @PostConstruct
    private void init() throws IOException {
        GoogleCredentials credentials =
                GoogleCredentials.fromStream(
                        this.getClass().getClassLoader().getResourceAsStream(serviceAccountKey));

        this.adminServiceSettings =
                AnalyticsAdminServiceSettings.newBuilder()
                        .setCredentialsProvider(FixedCredentialsProvider.create(credentials))
                        .build();
    }

    public List<GAAccountDTO> listGAAccounts() throws DataException {
        List<GAAccountDTO> gaAccounts = Lists.newArrayList();
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // Initializes the list request with default settings.
            ListAccountsRequest request = ListAccountsRequest.getDefaultInstance();

            for (Account account : analyticsAdminServiceClient.listAccounts(request).iterateAll()) {
                GAAccountDTO gaAccount = new GAAccountDTO();
                gaAccount.setId(parseId(account.getName()));
                gaAccount.setName(account.getName());
                gaAccount.setDisplayName(account.getDisplayName());
                gaAccount.setCreateTime(account.getCreateTime().getSeconds());
                gaAccount.setUpdateTime(account.getUpdateTime().getSeconds());

                // Adds into the list;
                gaAccounts.add(gaAccount);
            }

            log.info("listGAAccounts : {}", gson.toJson(gaAccounts));
            return gaAccounts;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    public List<GAPropertyDTO> listGAProperties(String accountName, boolean withDataStreams)
            throws DataException {
        List<GAPropertyDTO> gaProperties = Lists.newArrayList();
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // Initializes the list request with account name filter.
            // See
            // https://developers.google.com/analytics/devguides/config/admin/v1/rest/v1beta/properties/list#http-request
            ListPropertiesRequest request =
                    ListPropertiesRequest.newBuilder().setFilter("ancestor:" + accountName).build();

            for (Property property :
                    analyticsAdminServiceClient.listProperties(request).iterateAll()) {
                GAPropertyDTO gaProperty = new GAPropertyDTO();
                gaProperty.setId(parseId(property.getName()));
                gaProperty.setName(property.getName());
                gaProperty.setDisplayName(property.getDisplayName());
                gaProperty.setPropertyType(property.getPropertyType().toString());
                gaProperty.setAccount(property.getAccount());
                gaProperty.setParent(property.getParent());
                gaProperty.setIndustryCategory(property.getIndustryCategory().toString());
                gaProperty.setTimeZone(property.getTimeZone());
                gaProperty.setCurrencyCode(property.getCurrencyCode());
                gaProperty.setServiceLevel(property.getServiceLevel().toString());
                gaProperty.setCreateTime(property.getCreateTime().getSeconds());
                gaProperty.setUpdateTime(property.getUpdateTime().getSeconds());

                // Sets the data streams associated with this property.
                if (withDataStreams) {
                    gaProperty.setDataStreams(listGADataStreams(property.getName()));
                }

                // Adds into the list;
                gaProperties.add(gaProperty);
            }

            log.info("listGAProperties for {} : {}", accountName, gson.toJson(gaProperties));
            return gaProperties;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    public List<GADataStreamDTO> listGADataStreams(String propertyName) throws DataException {
        List<GADataStreamDTO> gaDataStreams = Lists.newArrayList();
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // Initializes the list request with parent property.
            ListDataStreamsRequest request =
                    ListDataStreamsRequest.newBuilder().setParent(propertyName).build();

            for (DataStream dataStream :
                    analyticsAdminServiceClient.listDataStreams(request).iterateAll()) {
                GADataStreamDTO gaDataStream = new GADataStreamDTO();
                gaDataStream.setId(parseId(dataStream.getName()));
                gaDataStream.setName(dataStream.getName());
                gaDataStream.setDisplayName(dataStream.getDisplayName());
                gaDataStream.setStreamMeasurementId(
                        dataStream.getWebStreamData().getMeasurementId());
                gaDataStream.setStreamDefaultUrl(dataStream.getWebStreamData().getDefaultUri());

                // Adds into the list;
                gaDataStreams.add(gaDataStream);
            }

            log.info("listGADataStreams for {} : {}", propertyName, gson.toJson(gaDataStreams));
            return gaDataStreams;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    /** Parses GA ID from the resource name. */
    private static String parseId(String name) {
        List<String> parts = Splitter.on('/').splitToList(name);
        return parts.size() == 2 ? parts.get(1) : null;
    }
}
