package com.wechi.adweb.bridge.google.analytics;

import static com.wechi.adweb.bridge.google.analytics.dto.GAReportRequestDTO.*;

import com.google.analytics.data.v1beta.*;
import com.google.api.gax.core.FixedCredentialsProvider;
import com.google.auth.oauth2.GoogleCredentials;
import com.google.common.base.Splitter;
import com.google.gson.Gson;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.GAReportRequestDTO;

import jakarta.annotation.PostConstruct;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;

import java.io.IOException;
import java.util.List;
import java.util.Objects;

@Slf4j
@Data
public class GADataService {

    @Value("${spring.application.name}")
    private String applicationName;

    @Value("${google.service.account.key}")
    private String serviceAccountKey;

    private BetaAnalyticsDataSettings dataSettings;

    private Gson gson = new Gson();

    @PostConstruct
    private void init() throws IOException {
        GoogleCredentials credentials =
                GoogleCredentials.fromStream(
                        this.getClass().getClassLoader().getResourceAsStream(serviceAccountKey));

        this.dataSettings =
                BetaAnalyticsDataSettings.newBuilder()
                        .setCredentialsProvider(FixedCredentialsProvider.create(credentials))
                        .build();
    }

    List<Object> runReport(GAReportRequestDTO reportRequest) throws DataException {
        try (BetaAnalyticsDataClient analyticsDataClient =
                BetaAnalyticsDataClient.create(dataSettings)) {
            // 1. Dimensions.
            List<Dimension> dimensions =
                    Splitter.on(GAReportRequestDTO.DEFAULT_SPLITTER)
                            .splitToStream(StringUtils.defaultString(reportRequest.getDimensions()))
                            .filter(StringUtils::isNotEmpty)
                            .map(dimension -> Dimension.newBuilder().setName(dimension).build())
                            .toList();

            // 2. Metrics.
            List<Metric> metrics =
                    Splitter.on(GAReportRequestDTO.DEFAULT_SPLITTER)
                            .splitToStream(StringUtils.defaultString(reportRequest.getMetrics()))
                            .filter(StringUtils::isNotEmpty)
                            .map(metric -> Metric.newBuilder().setName(metric).build())
                            .toList();

            // 3. Date range.
            DateRange dateRange =
                    DateRange.newBuilder()
                            .setStartDate(reportRequest.getStartDate())
                            .setEndDate(reportRequest.getEndDate())
                            .build();

            // 4. Order by.
            OrderBy.Builder orderByBuilder = OrderBy.newBuilder();

            String orderByName = reportRequest.getOrderBy();
            OrderByType orderByType = reportRequest.getOrderByType();
            if (StringUtils.isNotEmpty(orderByName) && Objects.nonNull(orderByType)) {
                switch (orderByType) {
                    case DIMENSIONS ->
                            orderByBuilder.setDimension(
                                    OrderBy.DimensionOrderBy.newBuilder()
                                            .setDimensionName(orderByName));
                    case METRICS ->
                            orderByBuilder.setMetric(
                                    OrderBy.MetricOrderBy.newBuilder().setMetricName(orderByName));
                }

                // Desc.
                orderByBuilder.setDesc(reportRequest.isOrderByDesc());
            }

            RunReportRequest request =
                    RunReportRequest.newBuilder()
                            .setProperty(reportRequest.getPropertyName())
                            .addAllDimensions(dimensions)
                            .addAllMetrics(metrics)
                            .addDateRanges(dateRange)
                            .addOrderBys(orderByBuilder)
                            // The API returns a maximum of 250,000 rows per request. See
                            // https://developers.google.com/analytics/devguides/reporting/data/v1/rest/v1beta/properties/runReport
                            .setLimit(250_000L)
                            .build();

            RunReportResponse response = analyticsDataClient.runReport(request);

            // TODO: Parses the response.
            return null;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }
}
